package jp.agentec.sinaburocast.common.interceptor;

import java.lang.annotation.Annotation;
import java.util.List;

import jp.agentec.sinaburocast.common.annotation.LogLevelInfo;
import jp.agentec.sinaburocast.common.annotation.SkipTrace;
import jp.agentec.sinaburocast.common.util.PropertyUtil;

import org.aopalliance.intercept.MethodInvocation;
import org.seasar.framework.aop.annotation.SimpleTrace;
import org.seasar.framework.log.Logger;

/**
 * Trace Interceptorクラス<br>
 * ・pointcutのカスタマイズ
 * ・メソッドの引数を終了時に出力しない
 * ・SkipTraceアノテーションの際、トレース出力しない
 * ・クラス名をシンプル名に変更
 *
 * @author tsukada
 *
 */
public class TraceInterceptor extends org.seasar.framework.aop.interceptors.TraceInterceptor {
    private static final long serialVersionUID = -9197894563727449743L;

    private static final Logger logger = Logger.getLogger(TraceInterceptor.class);
    
    private static List<String> noDebugList = PropertyUtil.getList("NO_DEBUG_CLASS_LIST"); // デバッグしないリスト

	public TraceInterceptor() {
	}

    @Override
    public Object invoke(final MethodInvocation invocation) throws Throwable {
    	// add SkipTrace annotation
        Annotation skipTrace = invocation.getMethod().getAnnotation(SkipTrace.class);
        if (skipTrace != null) {
            return invocation.proceed();
        }

        boolean simple = (invocation.getMethod().getAnnotation(SimpleTrace.class) != null);
        boolean info = (invocation.getMethod().getAnnotation(LogLevelInfo.class) != null);

        if (!logger.isDebugEnabled() && !info) {
            return invocation.proceed();
        }
        
        if (noDebugList.contains(getTargetClass(invocation).getSimpleName())) { // デバッグしないリストに含まれる場合
            return invocation.proceed();
        }
        
        final StringBuffer buf = new StringBuffer(100);
        buf.append(getTargetClass(invocation).getSimpleName()); // simple name
        buf.append("#").append(invocation.getMethod().getName());
        buf.append("(");
        if (!simple) {
            final Object[] args = invocation.getArguments();
            if (args != null && args.length > 0) {
                for (int i = 0; i < args.length; ++i) {
                    appendObject(buf, args[i]).append(", ");
                }
                buf.setLength(buf.length() - 2);
            }
        }
        buf.append(")");
        Object ret = null;
        Throwable cause = null;
        if (info) {
            logger.info("BEGIN " + buf);
        }
        else {
            logger.debug("BEGIN " + buf);
        }
        try {
            ret = invocation.proceed();
            buf.delete(0, buf.length()); // just output ret at END
            buf.append(getTargetClass(invocation).getSimpleName());
            buf.append("#").append(invocation.getMethod().getName()).append(":");
            if (!simple) {
                appendObject(buf, ret);
            }
        } catch (final Throwable t) {
            buf.append(" Throwable:").append(t);
            cause = t;
        }
        if (info) {
            logger.info("END " + buf);
        }
        else {
            logger.debug("END " + buf);
        }
        if (cause == null) {
            return ret;
        }
        throw cause;
    }

}
